#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import os
from ssrs.SSRS import SSRS

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

Service	  = 'http://servername/reportserver/ReportService2010.asmx?WSDL'
Execution = 'http://servername/reportserver/ReportExecution2005.asmx?WSDL'
user	  = 'username'
password  = 'password'
domain    = 'STRUCTURETEC'
RS		= SSRS(Service, Execution, user, password, domain)
RenderedReport =  None

try:
	Report = RS.RequestReport(path='/DMS Reports/BEP Executive Five-Year Plan')
except Exception as e:
	print('Error loading report: %s' %e.args)

try:
	# Put the parameters into a dictionary
	Parameters = {
		'clients': 'BASF',
		'projno' : 'T18087' 
	}
	
	RenderedReport = RS.RenderReport(LoadedReport=Report, format='PDF', parameters=Parameters)
except Exception as e:
	print('Error rendering report: %s' %e.args)


filename = os.path.join(os.path.dirname(__file__),'report'+ '.' + RenderedReport['Extension'])
fopen = open(filename, 'wb')
fopen.write(RenderedReport['Result'])

'''
	Testing GetParameters
'''
result	= RS.GetParameters('/DMS Reports/BEP Executive WCI Report')

for name in result :
	print('Name:', name)

'''
	Testing Find
'''
result = RS.Find(name='BEP Building Exec Summary', objtype='Report')

for key, value in result.items() :
		print('ID:', key)
		
		for key2, value2 in value.items():
			print(key2,':', value2)
		
		print('\n')

Methods = RS.ListServiceMethods()

if Methods:
    print(
        '''
        List of RPC Methods available on this SSRS version 
        Please see the MS docs to find out their usability
        '''
    )
    
    for i in Methods:
        print('>> ', i)

results = RS.ListExtensions('/DMS Reports/BEP Executive WCI Report')

for key, value in results.items() :
	print (value['Name'])



# -*- coding: utf-8 -*-


xml_Execute_Report_Parameter = '''
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:rep="http://schemas.microsoft.com/sqlserver/2005/06/30/reporting/reportingservices">
   <soapenv:Header>
      <rep:ExecutionHeader>
         <rep:ExecutionID>{0}</rep:ExecutionID>
      </rep:ExecutionHeader>
   </soapenv:Header>
   <soapenv:Body>
      <rep:SetExecutionParameters>
         <rep:Parameters>
        {1}
         </rep:Parameters>
         <rep:ParameterLanguage>en-US</rep:ParameterLanguage>
      </rep:SetExecutionParameters>
   </soapenv:Body>
</soapenv:Envelope>
'''


xml_Render_Report ='''<soapenv:Envelope 
xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" 
xmlns:rep="http://schemas.microsoft.com/sqlserver/2005/06/30/reporting/reportingservices">
<soapenv:Header>
<rep:ExecutionHeader>
<rep:ExecutionID>{0}</rep:ExecutionID>
</rep:ExecutionHeader>
</soapenv:Header>
<soapenv:Body>
<rep:Render>
<rep:Format>{1}</rep:Format>
</rep:Render>
</soapenv:Body>
</soapenv:Envelope>'''


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import connection_data as d
import context
from SSRS import SSRS

RS      = SSRS(d._service, d._execution, d._user, d._psw)
Methods = RS.ListMethods()

if Methods:
    print(
        '''
        List of RPC Methods available on this SSRS version 
        Please see the MS docs to find out their usability
        '''
    )
    
    for i in Methods:
        print('>> ', i)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import connection_data as d
import context
from SSRS import SSRS

RS      = SSRS(d._service, d._execution, d._user, d._psw)
result  = RS.GetParameters('/Qualidade/Indice de Conformidade')

print(
        '''
        Get Parameters linked to a specific report.
        
        Available Parameters:
                ParameterTypeName
                Nullable
                AllowBlank      
                MultiValue      
                QueryParameter  
                Prompt       
                PromptUser
                ValidValuesQueryBased
                DefaultValuesQueryBased
        '''
    )

for key, value in result.items() :
    print('Name:', key)
    
    for key2, value2 in value.items():
        print(key2,':', value2)
    
    print('\n')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import connection_data as d
import context
from SSRS import SSRS

RS      = SSRS(d._service, d._execution, d._user, d._psw)
BaseDirObjects = RS.ListDirItems()

if BaseDirObjects:
    print(
        '''           
        List of Objects in the root container (/)
        
        -> To check other directorys use the <dir> parameter
        -> To retrieve the directory and subdirectory's objects use <recursive=True> as a param
        
        Available information:
           ID 
           Name 
           Path
           TypeName
           CreationDate
           ModifiedDate
           CreatedBy
           ModifiedBy
           ItemMetadata

        Examples of usage:
            BaseDirObjects = RS.ListDirItems()
            BaseDirObjects = RS.ListDirItems(dir='/Customers')
            BaseDirObjects = RS.ListDirItems(recursive=True)
        '''
    )
    for key, value in BaseDirObjects.items() :
        print('ID:', key)
    
        for key2, value2 in value.items():
            print(key2,':', value2)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import connection_data as d
import context
from SSRS import SSRS

RS     = SSRS(d._service, d._execution, d._user, d._psw)
result = RS.Find(text='Lote', objtype='Report')

if result:
    print(
        '''
        This method will search for any property value that matches with the parameter <text>
        It'll match if any property value of the object CONTAINS the <text> value
        
        You can also specify the type of the object by using the <objtype> parameter as a string
        
        Types accepted:
            -> Component 
            -> DataSource 
            -> Folder
            -> Model
            -> LinkedReport
            -> Report
            -> Resource
            -> DataSet
            -> Site
            -> Unknown
            
        if you leave the <text> parameter blank it'll retrieve all objects (respecting the <objtype> parameter)
        '''
    )
    
    for key, value in result.items() :
        print('ID:', key)
        
        for key2, value2 in value.items():
            print(key2,':', value2)
        
        print('\n')


# -*- coding: utf-8 -*-

from suds.client import Client
from suds.sax.text import Text
from suds.sax.element import Element
from suds.transport.http import HttpAuthenticated
import requests
from requests_ntlm import HttpNtlmAuth
import suds_requests
from . import schemas
import base64
import logging as log
import codecs
import uuid
import sys
import os
from _ast import Load

class SSRS():
	'''
	Create a SOAP connection to a SSRS (Microsoft Reporting Services)

	Example of usage on SSRS 2008:
		RS = SSRS(ReportService	  = 'http://localhost/ReportinServices/ReportService2010.asmx?wsdl', 
				  ReportExecution = 'http://myserver/reportserver/ReportExecution2005.asmx?wsdl',
				  user			  = 'user@domain.com', 
				  key_password	  = 'myfreakingpassword'
				  domain		  = 'mydomain'
				)

	'''

	def __init__(self, ReportService, ReportExecution, user, key_password, domain=None, verbose=True):
		
		self.verbose = verbose
		servsession = requests.Session()
		execsession = requests.Session()
		log.basicConfig(filename = 'SSRS.log', level = log.INFO)

		if domain:
			user = '{}\\{}'.format(domain, user)
			servsession.auth = HttpNtlmAuth(user, key_password)
			execsession.auth = HttpNtlmAuth(user, key_password)
		else: 
			servsession.auth = (user, key_password)
			execsession.auth = (user, key_password)

		try:
			self.ServiceClient = Client(ReportService, transport = suds_requests.RequestsTransport(servsession))
			self.ExecutionClient = Client(ReportExecution, transport = suds_requests.RequestsTransport(execsession))
		except Exception as e:		  
			msg = "Error during connection: %s" % e.args
			log.error(msg)			
		
			if self.verbose: 
				print(msg)
				
			exit() 

	def ListServiceMethods(self):
		'''
		Return the list of methods available for your SSRS Service version
		'''
		try:
			list_of_methods = [method for method in self.ServiceClient.wsdl.services[0].ports[0].methods]
			return list_of_methods

		except Exception as e:
			msg = "ListServiceMethods() Could not retrieve the methods: %s" % e.args	 
			log.error(msg)			  
			
			if self.verbose: 
				print(msg)
				
			return False
	
	def ListExecutionMethods(self):
		'''
		Return the list of methods available for your SSRS Execution version
		'''
		try:
			list_of_methods = [method for method in self.ExecutionClient.wsdl.services[0].ports[0].methods]
			return list_of_methods

		except Exception as e:
			msg = "ListExecutionMethods() Could not retrieve the methods: %s" % e.args	 
			log.error(msg)			  
			
			if self.verbose: 
				print(msg)
				
			return False

	def ListDirItems(self, dir = r'/', recursive=False):
		'''
		List all itens in a folder. 
		if specified the <recursive> parameter, subfolders will be scanned too
		'''
		try:
			it = self.ServiceClient.service.ListChildren(dir, recursive)
			it = it.CatalogItem
		except Exception as e:
			msg = "ListDirItems() Could not retrieve the Objects: %s" % e.args
			log.error(msg)
			
			if self.verbose:	 
				print(msg)
				
			return False

		catalog_dict = {}
		
		for item in it:
			catalog_dict[item['ID']] = {
					'Name'			: item['Name'],
					'Path'			: item['Path'],
					'TypeName'		: item['TypeName'],
					'CreationDate'	: item['CreationDate'],
					'ModifiedDate'	: item['ModifiedDate'],
					'CreatedBy'		: item['CreatedBy'],
					'ModifiedBy'	: item['ModifiedBy'],
					'ItemMetadata'	: item['ItemMetadata'],
				}
		
		return catalog_dict

	def Find(self, name, objtype = None):		
		'''
		Find objects on SSRS 
		'''
		try: 
			it = self.ListDirItems(recursive = True)
		except Exception as e:
			msg = "Find() Could not retrieve the Objects: %s" % e.args
			log.error(msg)
			
			if self.verbose:	 
				print(msg)	
				
			return
		
		catalog_dict = {}
		for key, value in it.items() :
			for word in value.values():
				if name in str(word):
					if objtype == None:
						catalog_dict[key] = value
					elif it[key]['TypeName'] == objtype:
						catalog_dict[key] = value
		
		return catalog_dict

	def GetParameters(self, path):
		'''
		Retrieve parameters from an SSRS Report
		'''
		try:
			it = self.ServiceClient.service.GetItemParameters(path, None, True, None, None)
		except Exception as e:
			msg = "GetParameters() Could not retrieve the parameters: %s" % e.args
			log.error(msg)
			if self.verbose:
				print(msg)
				return
			  
		param_dict = {}
		for item in it.ItemParameter:
			param_dict[item['Name']] = {
				 'ParameterTypeName'		: item['ParameterTypeName'],
				 'Nullable'					: item['Nullable'],
				 'AllowBlank'				: item['AllowBlank'],
				 'MultiValue'				: item['MultiValue'],
				 'QueryParameter'			: item['QueryParameter'],
				 'Prompt'					: item['Prompt'],
				 'PromptUser'				: item['PromptUser'],
				 'ValidValuesQueryBased'	: item['ValidValuesQueryBased'],
				 'DefaultValuesQueryBased'	: item['DefaultValuesQueryBased'],
			}
		
		return param_dict

	def ListExtensions(self, path):
		'''
		Retrieve Usable Render Extensions
		'''
		try:
			it = self.ServiceClient.service.ListExtensions("Render")
		except Exception as e:
			msg = "ListExtensions() Could not retrieve the Extensions: %s" % e.args
			log.error(msg)
			if self.verbose:
				print(msg)
				return
		
		ext_dict = {}
		for item in it.Extension:
			ext_dict[item['Name']] = {
				 'Name'							: item['Name'],
				 'LocalizedName'				: item['LocalizedName'],
				 'Visible'						: item['Visible'],
				 'IsModelGenerationSupported'	: item['IsModelGenerationSupported'],
			}

		return ext_dict

	def RequestReport(self, path):
		try:
			return self.ExecutionClient.service.LoadReport(path, None)
		except Exception as e:
			msg = "Could not Load the Report: %s" % e.args
			log.error(msg)
			
			if self.verbose:	 
				print(msg)
			
			return
	
	def RenderReport(self, LoadedReport, format, **parameters):
		'''
		Render an already executed Report
		'''
		# check if format is valid
		available_formats = ['XML','NULL','CSV','IMAGE','PDF','HTML4.0', 'HTML3.2','MHTML','EXCEL','Word']

		if format not in available_formats:
			msg = "Format is not valid: %s" % format
			log.error(msg)
			
			if self.verbose:	 
				print(msg)
			
			return
		
		# Set parameters
		params = ''
		for k, v in parameters['parameters'].items():
			params =  params +'''
			<rep:ParameterValue>
			   <rep:Name>%s</rep:Name>
			   <rep:Value>%s</rep:Value>
			</rep:ParameterValue>
			''' % (k, v)
		
		param_xml = schemas.xml_Execute_Report_Parameter  
		param_xml = param_xml.format(LoadedReport.ExecutionID, params)
		
		try:
			setparam = self.ExecutionClient.service.SetExecutionParameters(__inject={'msg': param_xml})
		except Exception as e:
			msg = "Could not Send Parameters: %s" % e.args
			log.error(msg)
			
			if self.verbose:	 
				print(msg) 
			
			return
		
		# Default XML Schema | SUDS Factory doesent worked very well in this case
		xml = schemas.xml_Render_Report.format(LoadedReport.ExecutionID, format)
		
		# Render the report by its ExecutionID
		try:
			result = self.ExecutionClient.service.Render(__inject={'msg': xml})
		except Exception as e:
			msg = "Could not Render the Report: %s" % e.args
			log.error(msg)
			
			if self.verbose:	 
				print(msg) 
			
			return
		
		# Data to be sended
		Data = {}
		for k, v in result:
			if k == 'Result':
				 Data['Result'] = base64.b64decode(result.Result)
			else:
				Data[k] = v

		return Data
	
	def FlushCache(self, report):
		try:
			self.ServiceClient.service.FlushCache(report)
		except Exception as e:
			msg = "Could not Flush the Cache: %s" % e.args
			log.error(msg)
			
			if self.verbose:
				print(msg)

			return
	
	def FlushMultiCache(self, reports):
		try:
			for report in reports:
				self.ServiceClient.service.FlushCache(report)
		except Exception as e:
			msg = "Could not Flush the Cache: %s" % e.args
			log.error(msg)
			
			if self.verbose:
				print(msg)

			return
		
		
		


# -*- coding: utf-8 -*-

_service    = 'http://localhost/ReportinServices/ReportService2010.asmx?wsdl'
_execution  = 'http://localhost/ReportinServices/ReportExecution2005.asmx?wsdl'
_user       = 'user@domain.com'
_psw        = '@password2017'




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

requirements = ['suds-py3', 'requests', 'requests_ntlm', 'suds_requests4' ]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='pySSRS2',
    version='2.0.0',
    description=('Python SSRS integration'
                 'using SOAP RPCs'),
    long_description='Fixed version of pySSRS please see git for documentation',
    author="Fabricio Roberto reinert and Andrew Wheeler",
    author_email='genusistimelord@gmail.com',
    url='https://github.com/genusistimelord/PySSRS/',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='SSRS, Microsoft, Python, SOAP, RPC, Reporting, Services',
	python_requires='>=3.7',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
    ],
    test_suite='tests.test_app.tests',
    tests_require=test_requirements
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import connection_data as d
import context
import os
from SSRS import SSRS

RS = SSRS(d._service, d._execution, d._user, d._psw)

'''
Part 1
    This Section will load the report and it's data
    you can get parameters data, page size, and every info from it
'''
try:
    Report = RS.RequestReport(path='/Faturamento/Notas por Transportadora')

except BaseException as e:
    print('Error loadint report: %s' % str(e))


'''
Part 2
    This section will use the pre-loaded data to render the report
'''
try:
    # Put the parameters into a dictionary
    Parameters = {
        'Transp': -1,
        'Frete' : 'C' 
    }
    
    RenderedReport = RS.RenderReport(LoadedReport=Report, format='PDF', parameters=Parameters)

except BaseException as e:
    print('Error rendering report: %s' % str(e))


# This block will return all the information that SSRS can handle
for k, v in RenderedReport.items():
    if k == 'Result':
        pass
    
    else:
        print(k,': ',v)

'''
This block will render the report into a file
But you can use it in Django/Flask Request, for example...
    
    <> IMPORTANT - If you want to render a file <>
    Compiled files like PDF, Word, Excel should use "wb" on the file opening
    Text based files (xml, html...) are "OK" to use the "w" mode 
'''
        
filename = os.path.join(os.path.dirname(__file__),'report'+ '.' + RenderedReport['Extension'])
fopen = open(filename, 'wb')
fopen.write(RenderedReport['Result'])
    
    
